unit Unit_Client_Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ScktComp, ExtCtrls;

resourcestring
  StatusDateTimeFormat = 'mm/dd/yyyy" - "hh:nn:ss:zzz AM/PM';

type
  TForm_Client_Main = class(TForm)
    pcLearnSockets: TPageControl;
    tsClient: TTabSheet;
    gbBasicClientSettings: TGroupBox;
    btnClientClose: TButton;
    btnClientOpen: TButton;
    gbConnectionAddressOrHost: TGroupBox;
    rbtnConnectionAddress: TRadioButton;
    rbtnConnectionHost: TRadioButton;
    edConnectionAddress: TEdit;
    edConnectionHost: TEdit;
    gbConnectionportOrService: TGroupBox;
    rbtnConnectionService: TRadioButton;
    edConnectionPort: TEdit;
    edConnectionService: TEdit;
    rbtnConnectionPort: TRadioButton;
    gbClientType: TGroupBox;
    rbtnClientNonBlocking: TRadioButton;
    rbtnClientThreadBlocking: TRadioButton;
    lstClientActivity: TListBox;
    btnClearClientSocketLog: TButton;
    tsTextChat: TTabSheet;
    memSend: TMemo;
    ChatClientSocket: TClientSocket;
    memReceive: TMemo;
    edScreenName: TEdit;
    Label6: TLabel;
    lbUsers: TListBox;
    pnlRefreshList: TPanel;
    cbAutoRefreshUserList: TCheckBox;
    procedure btnClientOpenClick(Sender: TObject);
    procedure btnClientCloseClick(Sender: TObject);
    procedure btnClearClientSocketLogClick(Sender: TObject);
    procedure ChatClientSocketConnect(Sender: TObject; Socket: TCustomWinSocket);
    procedure ChatClientSocketConnecting(Sender: TObject; Socket: TCustomWinSocket);
    procedure ChatClientSocketDisconnect(Sender: TObject; Socket: TCustomWinSocket);
    procedure ChatClientSocketError(Sender: TObject; Socket: TCustomWinSocket; ErrorEvent: TErrorEvent; var ErrorCode: Integer);
    procedure ChatClientSocketLookup(Sender: TObject; Socket: TCustomWinSocket);
    procedure ChatClientSocketRead(Sender: TObject; Socket: TCustomWinSocket);
    procedure ChatClientSocketWrite(Sender: TObject; Socket: TCustomWinSocket);
    procedure memSendKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure memReceiveKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure pnlRefreshListClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form_Client_Main: TForm_Client_Main;

implementation

{$R *.DFM}

procedure TForm_Client_Main.btnClientOpenClick(Sender: TObject);
begin
  //Set Connection Port or Service
  if (rbtnConnectionPort.Checked) then
    ChatClientSocket.Port := StrToInt(edConnectionPort.Text)
  else if (rbtnConnectionService.Checked) then
    ChatClientSocket.Service := edConnectionPort.Text;
  //Set Connection Address or Host
  if (rbtnConnectionAddress.Checked) then
    ChatClientSocket.Address := edConnectionAddress.Text
  else if (rbtnConnectionHost.Checked) then
    ChatClientSocket.Host := edConnectionHost.Text;
  //Set Client Connection Type
//  if (rbtnClientNonBlocking.Checked) then
    ChatClientSocket.ClientType := ctNonBlocking;
//  if (rbtnClientThreadBlocking.Checked) then
//    ChatClientSocket.ClientType := ctBlocking;
  //Open Connection
  lstClientActivity.Items.Add('Client: Opening...');
  ChatClientSocket.Open;
  lstClientActivity.Items.Add('Client: Open');
  lstClientActivity.Items.Add('Local Host: ' + ChatClientSocket.Socket.LocalHost);
  lstClientActivity.Items.Add('Local Address: ' + ChatClientSocket.Socket.LocalAddress);
  lstClientActivity.Items.Add('Local Port: ' + IntToStr(ChatClientSocket.Socket.LocalPort));
end;

procedure TForm_Client_Main.btnClientCloseClick(Sender: TObject);
begin
  lstClientActivity.Items.Add('Client: Closing...');
  ChatClientSocket.Close;
  memReceive.Lines.Append('Closed(' + FormatDateTime(StatusDateTimeFormat, Now) + ')');  
  lstClientActivity.Items.Add('Client: Closed');
end;

procedure TForm_Client_Main.ChatClientSocketConnect(Sender: TObject; Socket: TCustomWinSocket);
begin
  lstClientActivity.Items.Add('Client: Connected');
  memReceive.Lines.Append('Connected...(' + FormatDateTime(StatusDateTimeFormat, Now) + ')' + #13#10);
end;

procedure TForm_Client_Main.ChatClientSocketConnecting(Sender: TObject; Socket: TCustomWinSocket);
begin
  lstClientActivity.Items.Add('Client: Connecting...');
end;

procedure TForm_Client_Main.ChatClientSocketDisconnect(Sender: TObject; Socket: TCustomWinSocket);
begin
  lstClientActivity.Items.Add('Client: Disconnecting...');
  memReceive.Lines.Append('Disconnected...(' + FormatDateTime(StatusDateTimeFormat, Now) + ')');
  pnlRefreshList.Color := clGreen;  
  lstClientActivity.Items.Add('Client: Disconnected');
end;

procedure TForm_Client_Main.ChatClientSocketError(Sender: TObject; Socket: TCustomWinSocket; ErrorEvent: TErrorEvent; var ErrorCode: Integer);
begin
  lstClientActivity.Items.Add('Client Error: #' + IntToStr(ErrorCode));
//ErrorExents
//eeGeneral	   //The socket received an error message that does not fit into any of the following categories.
//eeSend	     //An error occurred when trying to write to the socket connection.
//eeReceive	   //An error occurred when trying to read from the socket connection.
//eeConnect	   //A connection request that was already accepted could not be completed.
//eeDisconnect //An error occurred when trying to close a connection.
//eeAccept	   //A problem occurred when trying to accept a client connection request.
end;

procedure TForm_Client_Main.ChatClientSocketLookup(Sender: TObject; Socket: TCustomWinSocket);
begin
  lstClientActivity.Items.Add('Client: Lookup');
end;

procedure TForm_Client_Main.ChatClientSocketRead(Sender: TObject; Socket: TCustomWinSocket);
type
  PrevUser = record
    User : string;
    Selected : Boolean;
  end;
  PrevUserList = array of PrevUser;
var
  ReadText : string;
  Temp : PrevUserList;
  I, J : Integer;
begin
  ReadText := Socket.ReceiveText;
  lstClientActivity.Items.Add('Client: Read ' + FormatDateTime(StatusDateTimeFormat, Now));
  if (Copy(ReadText, 1, 1) = '"') then //A list of users was sent
    begin
      SetLength(Temp, lbUsers.Items.Count);
      for I := 0 to lbUsers.Items.Count - 1 do
      begin
        Temp[I].User := lbUsers.Items[I];
        Temp[I].Selected := lbUsers.Selected[I];
      end;
      lbUsers.Clear;
      lbUsers.Items.CommaText := Copy(ReadText, 1, Length(ReadText));
      //Reselect users
      for I := 0 to lbUsers.Items.Count - 1 do
      begin
        for J := 0 to High(Temp) do
        begin
          if (Temp[J].User = lbUsers.Items[I]) then
            if (Temp[J].Selected) then lbUsers.Selected[I] := True;
        end;
      end;
      pnlRefreshList.Color := clGreen;
    end
  else if (Copy(ReadText, 1 ,1) = '*') then
    begin
      pnlRefreshList.Color := clLime;
      if (cbAutoRefreshUserList.Checked) then pnlRefreshList.OnClick(Self);
    end
  else
    memReceive.Lines.Append(ReadText);
end;

procedure TForm_Client_Main.ChatClientSocketWrite(Sender: TObject; Socket: TCustomWinSocket);
begin
  lstClientActivity.Items.Add('Client: Write ' + FormatDateTime(StatusDateTimeFormat, Now));
  Socket.SendText('SName=' + edScreenName.Text);
  Application.ProcessMessages;
end;

procedure TForm_Client_Main.btnClearClientSocketLogClick(Sender: TObject);
begin
  lstClientActivity.Items.Clear;
end;

procedure TForm_Client_Main.memSendKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
var
  WriteText : string;
  Receivers : TStringList;
  I : Integer;
begin
  if (Key = VK_RETURN) then
  begin
    if (UpperCase(Copy(memSend.Text, 1 ,6)) = 'SNAME=') then
      begin
        WriteText := memSend.Text;
        edScreenName.text := Copy(memSend.Text, 7, Length(memSend.Text) - 6);
        lbUsers.Clear;
      end
    else if (Copy(memSend.Text, 1 ,1) = '"') then
      WriteText := '"'
    else if (Copy(memSend.Text, 1 ,1) = '?') then
      WriteText := '?'
    else
      begin
        //<Sender><Receiver, Receiver...Receiver><Message>
        Receivers := TStringList.Create;
        for I := 0 to lbUsers.Items.Count - 1 do
        begin
          if lbUsers.Selected[I] then Receivers.Add(lbUsers.Items[I]);
        end;
        WriteText := '<' + edScreenName.Text + '>' +
                     '<' + Receivers.CommaText + '>' +
                     '<' + memSend.Text + #13#10'>';
        Receivers.Free;
      end;
    if ChatClientSocket.Active then
      begin
        memReceive.Lines.Append(edScreenName.Text + ': ' + memSend.Text + #13#10);
        ChatClientSocket.Socket.SendText(WriteText);
      end
    else
        memReceive.Lines.Append('Cannot send message when disconnected!' + #13#10);
    memSend.Clear;
  end;
end;

procedure TForm_Client_Main.memReceiveKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if (key = VK_DELETE) then memReceive.Clear;
end;

procedure TForm_Client_Main.pnlRefreshListClick(Sender: TObject);
begin
  pnlRefreshList.Color := clGreen;
  ChatClientSocket.Socket.SendText('"');  
end;

end.
